@@ -715,7 +715,10 @@ xfce_sm_client_parse_argv(XfceSMClient *sm_client)
         for(i = 0; i < argc; ++i)
             restart_command[i] = argv[i];
         restart_command[i++] = "--display";
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
+        /* TODO: Move xfwm_make_display_name to libxfce4ui */
         restart_command[i++] = display_name = gdk_screen_make_display_name(gscreen);
+G_GNUC_END_IGNORE_DEPRECATIONS
     } else
         restart_command = argv;
 
@@ -1852,10 +1855,8 @@ xfce_sm_client_set_desktop_file(XfceSMClient *sm_client,
             g_set_application_name(name);
     }
 
-#if GTK_CHECK_VERSION(2, 16, 0)
     if(gtk_window_get_default_icon_name())
         has_default_icon = TRUE;
-#endif
 
     icon_list = gtk_window_get_default_icon_list();
     if(icon_list) {
